@@ -57,6 +57,19 @@ typedef struct
 	regex_t		regexv;
 }			RegPattern;
 
+typedef enum ProcessManagementModes
+{
+	PM_STATIC = 1,
+	PM_DYNAMIC
+}			ProcessManagementModes;
+
+typedef enum ProcessManagementSstrategies
+{
+	PM_STRATEGY_AGGRESSIVE = 1,
+	PM_STRATEGY_GENTLE,
+	PM_STRATEGY_LAZY
+}			ProcessManagementSstrategies;
+
 typedef enum NativeReplicationSubModes
 {
 	SLONY_MODE = 1,
@@ -203,14 +216,21 @@ typedef struct
 typedef struct
 {
 	ClusteringModes	backend_clustering_mode;	/* Backend clustering mode */
-	char	   *listen_addresses;	/* hostnames/IP addresses to listen on */
+	ProcessManagementModes	process_management;
+	ProcessManagementSstrategies process_management_strategy;
+	char	   **listen_addresses;	/* hostnames/IP addresses to listen on */
 	int			port;			/* port # to bind */
-	char	   *pcp_listen_addresses;	/* PCP listen address to listen on */
+	char	   **pcp_listen_addresses;	/* PCP listen address to listen on */
 	int			pcp_port;		/* PCP port # to bind */
-	char	   *socket_dir;		/* pgpool socket directory */
+	char	   **unix_socket_directories;		/* pgpool socket directories */
+	char		*unix_socket_group;			/* owner group of pgpool sockets */
+	int			unix_socket_permissions;	/* pgpool sockets permissions */
 	char	   *wd_ipc_socket_dir;	/* watchdog command IPC socket directory */
 	char	   *pcp_socket_dir; /* PCP socket directory */
-	int			num_init_children;	/* # of children initially pre-forked */
+	int			num_init_children;	/* Maximum number of child to
+										 * accept connections */
+	int			min_spare_children;		/* Minimum number of idle children */
+	int			max_spare_children;		/* Minimum number of idle children */
 	int			listen_backlog_multiplier;	/* determines the size of the
 											 * connection queue */
 	int			reserved_connections;	/* # of reserved connections */
@@ -305,6 +325,16 @@ typedef struct
 									 * greater than 0 to enable the
 									 * functionality. */
 
+	int		delay_threshold_by_time;	/* If the standby server delays more than
+										* delay_threshold_in_time, any query goes to the
+										* primary only. The unit is in seconds. 0
+										* disables the check. Default is 0.
+										* If delay_threshold_in_time is greater than 0,
+										* delay_threshold will be ignored.
+										* Note that health_check_period required to be
+										* greater than 0 to enable the
+										* functionality. */
+
 	bool		prefer_lower_delay_standby;
 
 	LogStandbyDelayModes log_standby_delay; /* how to log standby lag */
@@ -368,8 +398,6 @@ typedef struct
 	bool		log_statement;	/* logs all SQL statements */
 	bool		log_per_node_statement; /* logs per node detailed SQL
 										 * statements */
-	bool		notice_per_node_statement; /* logs notice message for per node detailed SQL
-										 * statements */
 	bool		log_client_messages;	/* If true, logs any client messages */
 	char	   *lobj_lock_table;	/* table name to lock for rewriting
 									 * lo_creat */
@@ -383,6 +411,9 @@ typedef struct
 
 	/* followings till syslog, does not exist in the configuration file */
 	int			num_reset_queries;	/* number of queries in reset_query_list */
+	int			num_listen_addresses;	/* number of entries in listen_addresses */
+	int			num_pcp_listen_addresses;	/* number of entries in pcp_listen_addresses */
+	int			num_unix_socket_directories;	/* number of entries in unix_socket_directories */
 	int			num_read_only_function_list;	/* number of functions in
 											 * read_only_function_list */
 	int			num_write_function_list;	/* number of functions in
@@ -566,7 +597,8 @@ typedef struct
 	int			pgpool_node_id;	/* pgpool (watchdog) node id */
 	WdNodesConfig wd_nodes;		/* watchdog lists */
 	char	   *trusted_servers;	/* icmp reachable server list (A,B,C) */
-	char	   *delegate_IP;	/* delegate IP address */
+	char	   *trusted_server_command;	/* Executes this command when upper servers are observed */
+	char	   *delegate_ip;	/* delegate IP address */
 	int			wd_interval;	/* lifecheck interval (sec) */
 	char	   *wd_authkey;		/* Authentication key for watchdog
 								 * communication */
